## -*- mode: autoconf -*-

##
## This file is part of the Cube software (http://www.scalasca.org)
##
## Copyright (c) 2015, 2016
## Forschungszentrum Juelich GmbH, Germany
##
## This software may be modified and distributed under the terms of
## a BSD-style license.  See the COPYING file in the package base
## directory for details.
##


##
## Internally used macros
##

dnl This macro compares the required interface version with the
dnl triplet common to libtool versioning 'current:revision:age'.
dnl
dnl Input:
dnl   $1 result variable to be set to "yes", "no", or "skipped"
dnl   $2 required interface version
dnl   $3 library interface version in format: current:revision:age
dnl
dnl Output:
dnl   The variable specified in $1 is modified.
dnl
AC_DEFUN([_AX_CHECK_CUBE_INTERFACE_VERSION],[
    m4_ifblank([$1], [m4_fatal([Missing result variable argument.])])
    m4_ifblank([$2], [m4_fatal([Missing required interface version argument.])])
    m4_ifblank([$3], [m4_fatal([Missing detected interface version argument.])])

    AC_MSG_CHECKING([interface version required])
    AC_MSG_RESULT([$2])
    
    cube_max_provided_interface_version=`echo $3 | cut -d : -f 1`
    cube_provided_age=`echo $3 | cut -d : -f 3`
    AC_MSG_CHECKING([interface version provided])
    AC_MSG_RESULT([$3])

    AC_MSG_CHECKING([whether provided interface is suitable])
    AS_IF([test "x${cube_max_provided_interface_version}" != "x" && test "x${cube_provided_age}" != "x"],
          [dnl perform the check
           AS_IF([test ${cube_max_provided_interface_version} -eq 0 && test ${cube_provided_age} -eq 0],
                 [dnl 0:0:0 indicates a development library. Skip check.
                  $1="skipped"
                 ],
                 [dnl do the math
                  AS_VAR_ARITH([cube_min_provided_interface_version], [${cube_max_provided_interface_version} - ${cube_provided_age}])
                  AS_IF([test $2 -le ${cube_max_provided_interface_version} && test $2 -ge ${cube_min_provided_interface_version}],
                        [$1="yes"],
                        [$1="no"])
                 ])
          ],
          [$1="no"])

    AC_MSG_RESULT(${$1})
    AS_IF([test "x${$1}" = "xskipped"],              
          AC_MSG_WARN([The Cube library found is an unversioned development library. You may experience compile-time and/or link-time errors.]))
])


dnl This macro performs queries to the config tool until variable is not-empty
dnl
dnl Input:
dnl   $1 prefix (lowercase)
dnl   $2 suffix (any case)
dnl   $3 comma separated list of query arguments to test
dnl
dnl Output:
dnl   A variable with all-uppercase name constructed from 'prefix_suffix' is
dnl   set to the result of the respective config-tool call. The variable is
dnl   also passed to AC_SUBST().
dnl
AC_DEFUN([_AX_CUBE_CONFIG_QUERY],[

    m4_ifblank([$1], [m4_fatal([Missing prefix argument.])])
    m4_ifblank([$2], [m4_fatal([Missing suffix argument.])])
    m4_ifblank([$3], [m4_fatal([Missing list of queries argument.])])

AC_MSG_CHECKING([value for m4_toupper($1_$2)])
    m4_foreach([_CUBE_QUERY_VAR], [$3], [
        AS_IF([test "x${m4_toupper($1_$2)}" = "x"],
              [dnl variable is unset; try next query
               cube_config_query="${cube_$1_config} --_CUBE_QUERY_VAR"
               cube_config_query_output=`$cube_config_query 2>/dev/null`
               AS_IF([test "x$cube_config_query_output" != "x"],
                     [dnl query was successful; set variable
                      m4_toupper($1_$2)="$cube_config_query_output"
                      AC_SUBST(m4_toupper($1_$2))
                      dnl AC_MSG_RESULT([${m4_toupper($1_$2)} (via --_CUBE_QUERY_VAR)])
                      AC_MSG_RESULT([${m4_toupper($1_$2)}])
                     ])
              ])
    ])
    AS_IF([test "x${m4_toupper($1_$2)}" = "x"],
          AC_MSG_RESULT([]))
])

dnl This macro checks for the availability of a config tool for the required component and
dnl validates the corresponding library interface version matches the requested interface
dnl version.
dnl
dnl Input:
dnl   $1 component short name (used as prefix to the config-tool and all corresponding variables)
dnl   $2 required library interface version
dnl   $3 configure option name (a more descriptive name used for the --with-<component> option)
dnl
dnl Output:
dnl   cube_$1_config    : the detected config tool
dnl   cube_$1_config_ok : either "yes", "no", or "skipped" depending on library interface
dnl
dnl The macro will check for the availablility of either '$1-config' or
dnl the deprecated 'cube-config'.
dnl
dnl The revision returned by the config tool must follow libtools naming 
dnl scheme 'current:revision:age'. The component found is accepted, iff
dnl current >= $2 && $2 >= (current - age).
dnl 
dnl If $with_val=yes, the config tool is searched in the PATH, if
dnl $with_val=no, the component is disabled, otherwise the config tool
dnl is searched in the provided path. 
dnl
AC_DEFUN([_AX_HAVE_CUBE_CONFIG_TOOL],[

    m4_ifblank([$1], [m4_fatal([The argument 1 may not be empty.])])
    m4_ifblank([$2], [m4_fatal([The argument 2 may not be empty.])])
    m4_ifblank([$3], [m4_fatal([The argument 3 may not be empty.])])

    AS_UNSET([cube_$1_config])
    AS_UNSET([cube_$1_config_ok])

    AS_IF([test "x$CUBEGUI_CUBELIB" != "x"],
    [cubegui_cubelib_=`dirname $CUBEGUI_CUBELIB`],[cubegui_cubelib_=])
    
    AC_ARG_WITH([$3],
        [AS_HELP_STRING([--with-$3[[=DIR]]],
            [Provide path to a pre-installed $1-config or cube-config. Autodetected if already in $PATH.])],
        [AS_IF([test "x$withval" != "xno"],
              [AC_PATH_PROGS([cube_$1_config],[$1-config cube-config],[],[$cubegui_cubelib_ $withval $withval/bin $PATH])])],
        [AC_PATH_PROGS([cube_$1_config],[$1-config cube-config],[], [$cubegui_cubelib_ $PATH])])

    AS_IF([test "x${cube_$1_config}" != "x"],
          [dnl config tool found, do further processing
           cube_$1_config_interface_version=`${cube_$1_config} --interface-version`
           _AX_CHECK_CUBE_INTERFACE_VERSION([cube_$1_config_ok], $2, ${cube_$1_config_interface_version})
          ],
          [cube_$1_config_ok="no"])

    AS_IF([test "x$cube_$1_config_ok}" != "no"],
          [dnl The library was found
           AC_DEFINE(AS_TR_CPP(m4_toupper([HAVE_$3])),[1],[Define if $3 support is available.])])
])

dnl This macro will search for a suitable config tool for the Cube library
dnl
dnl Input:
dnl   $1 required interface version
dnl
dnl Output:
dnl   Defines HAVE_CUBE_LIBRARY
dnl
AC_DEFUN([AX_WITH_CUBE_LIBRARY],[

    m4_ifblank([$1], [m4_fatal([$0: missing required library interface version])])

    _AX_HAVE_CUBE_CONFIG_TOOL([cubelib], $1, [cube-library])

    AS_IF([test "x${cube_cubelib_config_ok}" != "xno"],
          [dnl query variables
           _AX_CUBE_CONFIG_QUERY([cubelib], [cppflags],       [cppflags, cube-cxxflags, cxxflags])
           _AX_CUBE_CONFIG_QUERY([cubelib], [include],        [include, cube-include-path])
           _AX_CUBE_CONFIG_QUERY([cubelib], [ldflags],        [ldflags, cube-shared-ldflags, cube-ldflags])
           _AX_CUBE_CONFIG_QUERY([cubelib], [libs],           [libs, cube-shared-library, cube-library])
          ])
])

dnl This macro will search for a suitable config tool for the Cube plugin library
dnl
dnl Input:
dnl   $1 required interface version
dnl_
dnl Output:
dnl   Defines HAVE_CUBE_GUI
dnl
AC_DEFUN([AX_WITH_CUBE_GUI],[

    m4_ifblank([$1], [m4_fatal([$0: missing required library interface version])])

    _AX_HAVE_CUBE_CONFIG_TOOL([cubegui], $1, [cube-gui])

    AS_IF([test "x${cube_cubegui_config_ok}" != "xno"],
          [dnl query variables
           _AX_CUBE_CONFIG_QUERY([cubegui], [cubelib],        [cubelib-config])
           _AX_CUBE_CONFIG_QUERY([cubegui], [cppflags],       [cppflags, gui-cxxflags, cxxflags])
           _AX_CUBE_CONFIG_QUERY([cubegui], [include],        [gui-include-path, include])
           _AX_CUBE_CONFIG_QUERY([cubegui], [ldflags],        [gui-shared-ldflags, gui-ldflags, cube-ldflags, ldflags])
           _AX_CUBE_CONFIG_QUERY([cubegui], [libs],           [gui-shared-library, gui-library, cube-library, libs])
           _AX_CUBE_CONFIG_QUERY([cubegui], [qmake],          [qmake])
          ])
])

dnl This macro will search for a suitable config tool for the Cube writer library
dnl
dnl Input:
dnl   $1 required interface version
dnl
dnl Output:
dnl   Defines HAVE_CUBE_WRITER
dnl
AC_DEFUN([AX_WITH_CUBE_WRITER],[

    m4_ifblank([$1], [m4_fatal([$0: missing required library interface version])])

    _AX_HAVE_CUBE_CONFIG_TOOL([cubew], $1, [cube-writer])

    AS_IF([test "x${cube_cubew_config_ok}" != "xno"],
          [dnl query variables
           _AX_CUBE_CONFIG_QUERY([cubew], [cppflags],       [cppflags, cube-cxxflags, cxxflags])
           _AX_CUBE_CONFIG_QUERY([cubew], [include],        [cubew-include-path, include])
           _AX_CUBE_CONFIG_QUERY([cubew], [ldflags],        [cubew-ldflags, cube-ldflags, ldflags])
           _AX_CUBE_CONFIG_QUERY([cubew], [libs],           [cubew-library, cube-library, libs])
          ])
])

dnl This macro will search for a suitable config tool for the Cube Java reader library
dnl
dnl Input:
dnl   $1 required interface version
dnl
dnl Output:
dnl   Defines HAVE_CUBE_JAVA_READER
dnl
AC_DEFUN([AX_WITH_CUBE_JAVA_READER],[

    m4_ifblank([$1], [m4_fatal([$0: missing required library interface version])])

    _AX_HAVE_CUBE_CONFIG_TOOL([jcuber], $1, [cube-java-reader])

    AS_IF([test "x${cube_jcuber_config_ok}" != "xno"],
          [dnl query variables
           _AX_CUBE_CONFIG_QUERY([jcuber], [classpath],      [classpath])
           _AX_CUBE_CONFIG_QUERY([jcuber], [cube_extension], [cube-extension])
          ])
])


